from cleanfid import fid

from .features import Features


def compute_kid(a: Features, b: Features) -> float:
    return fid.kernel_distance(a.features, b.features)


def test_kid_directories():
    import numpy as np

    np.random.seed(123)
    reference_kid_score = fid.compute_kid(
        "tests/pixelart/dataset_a", "tests/pixelart/dataset_b"
    )
    np.random.seed(123)
    features_a = Features.from_directory("tests/pixelart/dataset_a")
    features_b = Features.from_directory("tests/pixelart/dataset_b")
    assert compute_kid(features_a, features_b) == reference_kid_score



from cleanfid import fid

from .features import Features


def compute_fid(a: Features, b: Features) -> float:
    return fid.frechet_distance(a.mu, a.sigma, b.mu, b.sigma)


def test_fid_directories():
    reference_fid_score = fid.compute_fid(
        "tests/pixelart/dataset_a", "tests/pixelart/dataset_b"
    )

    features_a = Features.from_directory("tests/pixelart/dataset_a")
    features_b = Features.from_directory("tests/pixelart/dataset_b")
    assert compute_fid(features_a, features_b) == reference_fid_score



from typing import Union
from pathlib import Path
import numpy as np
import torch

from cleanfid import fid

MODE = "clean"


class Features:
    features: np.array

    def __init__(self, features: np.array):
        self.features = features

    @property
    def mu(self):
        return np.mean(self.features, axis=0)

    @property
    def sigma(self):
        return np.cov(self.features, rowvar=False)

    @staticmethod
    def from_directory(
        path: Union[str, Path],
        batch_size=32,
        n_workers=12,
        device=torch.device("cuda"),
    ) -> "Features":
        feature_model = fid.build_feature_extractor(MODE, device)
        features = fid.get_folder_features(
            str(path),
            feature_model,
            num_workers=n_workers,
            batch_size=batch_size,
            device=torch.device("cuda"),
            mode=MODE,
            description=f"Extracting features for {Path(path).name} : ",
            verbose=True,
            custom_image_tranform=None,
        )
        return Features(features=features)
